// Basic String Operations - Mutable Strings

#import <Foundation/NSObject.h>
#import <Foundation/NSString.h>
#import <Foundation/NSAutoreleasePool.h>

int main (int argc, char *argv[])
{
	NSAutoreleasePool  *pool = [[NSAutoreleasePool alloc] init];
	NSString  *str1 = @"This is string A"; 
	NSString  *search, *replace;
	NSMutableString  *mstr;
	NSRange   substr;

	// Create immutable string from nonmutable

	mstr = [NSMutableString  stringWithString: str1];
	printf ("%s\n", [mstr cString]);

	// Insert characters

	[mstr insertString: @" mutable" atIndex: 7];
	printf ("%s\n", [mstr cString]);

	// Effective concatentation if insert at end

	[mstr insertString: @" and string B" atIndex: [mstr length]];
	printf ("%s\n", [mstr cString]);

	//  Or can use appendString directly

	[mstr appendString: @" and string C"];
	printf ("%s\n", [mstr cString]);

	// Delete substring based on range

	[mstr deleteCharactersInRange: NSMakeRange (16, 13)];
	printf ("%s\n", [mstr cString]);

	// Find range first and then use it for deletion

	substr = [mstr  rangeOfString: @"string B and "];
	
	if (substr.location != NSNotFound) {
		[mstr deleteCharactersInRange: substr];
		printf ("%s\n", [mstr cString]);
	}

	// Set the mutable string directly

	[mstr setString: @"This is string A"];
	printf ("%s\n", [mstr cString]);

	// Now let's replace a range of chars with another

	[mstr replaceCharactersInRange: NSMakeRange(8, 8) 
	      withString: @"a mutable string"];
	printf ("%s\n", [mstr cString]);
	
	// Search and replace

	search = @"This is";
	replace = @"An example of";

	substr = [mstr  rangeOfString: search];
	
	if (substr.location != NSNotFound) {
		[mstr replaceCharactersInRange: substr  withString: replace];
		printf ("%s\n", [mstr cString]);
	}

	// Search and replace all occurrences

	search = @"a";
	replace = @"X";
	
	substr = [mstr rangeOfString: search];

	while (substr.location != NSNotFound) {
		[mstr replaceCharactersInRange: substr  withString: replace];
		substr = [mstr rangeOfString: search];
	}

	printf ("%s\n", [mstr cString]);

	[pool release];
	return 0;
}
