// Basic file operations
// Assumes the existence of a file called "testfile"
// in the current working directory

#import <Foundation/NSObject.h>
#import <Foundation/NSString.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSAutoreleasePool.h>
#import <Foundation/NSDictionary.h>

int main (int argc, char *argv[])
{
	NSAutoreleasePool  *pool = [[NSAutoreleasePool alloc]  init];
	NSString           *fName = @"testfile";
	NSFileManager      *NSFm;
	NSDictionary	     *attr;

	// Need to create an instance of the file manager

	NSFm = [NSFileManager  defaultManager];

	// Let's make sure our test file exists first

	if ([NSFm fileExistsAtPath:  fName] == NO) {
		NSLog (@"File doesn't exist!\n");
		return 1;
	}

	// Now let's make a copy

	if ([NSFm copyPath:  fName toPath:  @"newfile"  handler:  nil] == NO) {
		NSLog (@"File copy failed!\n");
		return 2;
	}

	// Let's test to see if the two files are identical

	if ([NSFm contentsEqualAtPath:  fName andPath:  @"newfile"]  ==  NO) {
		NSLog (@"Files are not equal!\n");
		return 4;
	}

	// Now let's rename the copy

	if ([NSFm movePath:  @"newfile"  toPath:  @"newfile2"
	           handler: nil] == NO) {
		NSLog (@"File rename failed!\n");
		return 5;
	}

	// Get the size of newfile2

        if ((attr = [NSFm fileAttributesAtPath:  @"newfile2"
                                 traverseLink: NO]) == nil) {
              NSLog (@"Couldn't get file attributes!\n");
              return 6;
        }

	NSLog (@"File size is %i bytes\n",
            [[attr objectForKey: NSFileSize] intValue]);

	// And finally, let's delete the original file

	if ([NSFm removeFileAtPath:  fName handler:  nil] == NO) {
		NSLog (@"File removal failed!\n");
		return 7;
	}
	
	NSLog (@"All operations were successful!\n");

	[pool release];
	return 0;
}

