// Implement a basic copy utility

#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSAutoreleasePool.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSProcessInfo.h>

int main (int argc, char *argv[])
{
	NSAutoreleasePool  *pool = [[NSAutoreleasePool alloc] init];
	NSFileManager      *NSFm;
	NSString	   *source, *dest;
	BOOL		   isDir;
	NSProcessInfo      *proc = [NSProcessInfo processInfo];
	NSArray            *args = [proc arguments];

	NSFm = [NSFileManager  defaultManager];

	// Check for two arguments on the command line

	if ([args count] != 3) {   
		printf ("Usage: %s src dest\n", [[proc processName] cString]);
		return 1;
	}

	source = [args objectAtIndex: 1];
	dest = [args objectAtIndex: 2];
	

	// Make sure the source file can be read

	if ([NSFm  isReadableFileAtPath: source] == NO) {
		printf ("Can't read %s\n", [source cString]);
		return 2;
	}

	// See if the destination file is a directory
	// if it is, add the source to the end of the destination

	[NSFm fileExistsAtPath: dest isDirectory: &isDir];

	if (isDir == YES)
		dest = [dest stringByAppendingPathComponent: 
				[source lastPathComponent]];

	// Remove the destination file if it already exists

	[NSFm removeFileAtPath: dest handler: nil];

	
	// Okay, time to perform the copy

	if ([NSFm copyPath: source toPath: dest handler: nil] == NO) {
		printf ("Copy failed!\n");
		return 3;
	}
	
	printf ("Copy of %s to %s succeeeded!\n", [source cString],
		[dest cString]);

	[pool release];
	return 0;
}

