#import "AddressBook.h"

@implementation AddressBook;

// set up the AddressBook's name and to an empty book

-(id) initWithName: (NSString *) name
{
	self = [super init];

	if (self != nil) {	bookName = [name copy];
		book = [[NSMutableArray  alloc] init];

		// cleanup if the book doesn’t initialize
		if (book == nil) {
			[self autorelease];
			self = nil;
		}

	}

	return self;
}

-(void) setName: (NSString *) theName
{
	[bookName autorelease];

	bookName = [theName copy];
}

-(void) setBook: (NSArray *) theBook
{
[book autorelease];

	book = [[NSMutableArray  alloc] initWithArray: theBook];
}

-(void) addCard: (AddressCard *) theCard
{
	[book addObject: theCard];
}

-(NSMutableArray *) book
{
	return [[book mutableCopy] autorelease];
}

-(NSString *) bookName
{
	return [[bookName mutableCopy] autorelease];
}

-(void) removeCard: (AddressCard *) theCard
{
	[book removeObjectIdenticalTo:  theCard];
}

-(void) sort
{
 [book sortUsingSelector: @selector(compareNames:)];
}

	
// lookup address card by name – returns 1st exact match

-(AddressCard *) lookup: (NSString *) theName
{
	AddressCard *nextCard;

	int	i, elements;

	elements = [book count];

	for ( i = 0; i < elements; ++i) {
		nextCard = [book objectAtIndex: i];

		if ( [[nextCard name] caseInsensitiveCompare: theName]
                  == NSOrderedSame )
			return nextCard;
	}

	return nil;
}

-(int) entries
{
	return [book count];
}

-(void) list
{
	int	 i;
	AddressCard *theCard;
	int  elements = [book count];

	printf ("\n======== Contents of: %s =========\n", [bookName cString]);
	for ( i = 0; i < elements; ++i ) {
	   theCard = [book objectAtIndex: i];
           printf ("%-20s %-32s\n", [[theCard name] cString],
			[[theCard email] cString]);
	}

	printf ("====================================================\n\n");
}

-(void) dealloc
{
    [bookName release];
    [book release];
    [super dealloc];
}

// Methods for NSCoding protocol

-(void) encodeWithCoder: (NSCoder *) encoder
{
	if ( [encoder allowsKeyedCoding] ) {
	   [encoder encodeObject:bookName forKey: @"AddressBook bookName"];
	   [encoder encodeObject:book forKey: @"AddressBook book"];
	} else {
	   [encoder encodeObject: bookName];
	   [encoder encodeObject: book];
	}
}

-(id) initWithCoder: (NSCoder *) decoder
{
	if ( [decoder allowsKeyedCoding] ) {
 		bookName = [[decoder decodeObjectForKey:
			@"AddressBook bookName"] retain];
	 	book = [[decoder decodeObjectForKey: @"AddressBook book"] retain];
	} else {
	 	bookName = [[decoder decodeObject] retain];
	 	book = [[decoder decodeObject] retain];
	}

	return self;
}


// Method for NSCopying protocol

-(AddressBook *) copyWithZone: (NSZone *) zone
{
	AddressBook *newBook = [[self class] allocWithZone: zone];

	[newBook initWithName: bookName];
	[newBook setBook: book];

	return newBook;
}
@end

