#import <Foundation/NSObject.h>
#import <Foundation/NSAutoreleasePool.h>
#import <Foundation/NSString.h>
#import <Foundation/NSKeyedArchiver.h>
#import <Foundation/NSCoder.h>
#import <Foundation/NSData.h>
#import "AddressBook.h"


@interface Foo: NSObject <NSCoding>
{
    NSString *strVal;
    int       intVal;
    float    floatVal;
}

-(void) setAll: (NSString *) ss iVal: (int) ii fVal: (float) ff;

-(NSString *) strVal;
-(int) intVal;
-(float) floatVal;
@end

@implementation Foo;
-(void) setAll: (NSString *) ss iVal: (int) ii fVal: (float) ff
{
    strVal = ss;
    intVal = ii;
    floatVal = ff;
}

-(NSString *) strVal   { return strVal; }
-(int)        intVal   { return intVal; }
-(float)      floatVal { return floatVal; }


-(void) encodeWithCoder: (NSCoder *) encoder
{
   if ( [encoder allowsKeyedCoding] ) {
      [encoder encodeObject: strVal forKey: @"FoostrVal"];
      [encoder encodeInt: intVal forKey: @"FoointVal"];
      [encoder encodeFloat: floatVal forKey: @"FoofloatVal"];
   } else {
      [encoder encodeObject: strVal];
      [encoder encodeValueOfObjCType: @encode(int) at: &intVal];
      [encoder encodeValueOfObjCType: @encode(float) at: &floatVal];
   }
}

-(id) initWithCoder: (NSCoder *) decoder
{
   if ( [decoder allowsKeyedCoding] ) {
      strVal = [[decoder decodeObjectForKey: @"FoostrVal"] retain];
      intVal = [decoder decodeIntForKey: @"FoointVal"];
      floatVal = [decoder decodeFloatForKey: @"FoofloatVal"];
   } else {
      strVal = [[decoder decodeObject] retain];
      [decoder decodeValueOfObjCType: @encode(int) at: &intVal];
      [decoder decodeValueOfObjCType: @encode(float) at: &floatVal];
   }

    return self;
}
@end

int main (int argc, char *argv[])
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	Foo               *myFoo1 = [[Foo alloc] init];
	Foo               *myFoo2;
	NSMutableData     *dataArea;
	NSKeyedArchiver   *archiver;

	// Following from Program 19-6

	NSString	*aName = @"Julia Kochan";
	NSString	*aEmail = @"jewls337@axlc.com";
	NSString	*bName = @"Tony Iannino";
	NSString	*bEmail = @"tony.iannino@techfitness.com";
	NSString	*cName = @"Stephen Kochan";
	NSString	*cEmail = @"steve@kochan-wood.com";
	NSString	*dName = @"Jamie Baker";
	NSString	*dEmail = @"jbaker@kochan-wood.com";

	AddressCard *card1 = [[AddressCard alloc] init];
	AddressCard *card2 = [[AddressCard alloc] init];
	AddressCard *card3 = [[AddressCard alloc] init];
	AddressCard *card4 = [[AddressCard alloc] init];

	AddressBook	*myBook = [AddressBook alloc];

	// First set up four address cards

	[card1 setName: aName andEmail: aEmail];
	[card2 setName: bName andEmail: bEmail];
	[card3 setName: cName andEmail: cEmail];
	[card4 setName: dName andEmail: dEmail];

	myBook = [myBook initWithName: @"Steve's Address Book"];

	// Add some cards to the address book

	[myBook addCard: card1];	
	[myBook addCard: card2];	
	[myBook addCard: card3];	
	[myBook addCard: card4];	

	[myFoo1 setAll: @"This is the string" iVal: 12345 fVal: 98.6];

	// Set up a data area and connect it to an NSKeyedArchiver object
	dataArea = [NSMutableData data];
	
	archiver = [[NSKeyedArchiver alloc]
                          initForWritingWithMutableData: dataArea];

	// Now we can begin to archive objects
	[archiver encodeObject: myBook forKey: @"myaddrbook"];
	[archiver encodeObject: myFoo1 forKey: @"myfoo1"];
	[archiver finishEncoding];

	// Write the archived data are to a file
	if ( [dataArea writeToFile: @"myArchive" atomically: YES] == NO)
		printf ("Archiving failed!\n");

	[archiver release];

	[card1 release];
	[card2 release];
	[card3 release];
	[card4 release];
	[myBook release];
	[myFoo1 release];	
	[pool release];
	return 0;
}

